#!/usr/bin/env bash

# tar.sh - backup files via tar.
# Copyright (c) 2022 ge <https://nixhacks.net/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

src_tar() {
    # Backup local files with tar(1). Handle 'file' URI scheme.
    #
    # Usage: src::tar URI

    log "Run handler ${FUNCNAME[0]}()"

    local uri
    local src_path
    local dst_path
    local archive
    local exclude

    uri="$1"
    # shellcheck disable=SC2154
    dst_path="$__main_target_path"

    parse_uri "$uri"

    # shellcheck disable=SC2154
    if [[ "$scheme" != 'file' ]]; then
        log -p "Error: Wrong URI scheme: $scheme" >&2; exit 1
    fi

    # shellcheck disable=SC2154
    if [ -f "$path" ] || [ -d "$path" ]; then
        src_path="$path"
    else
        echo "Error: Path '$path' from URI '$uri' does not exists" >&2
        exit 1
    fi

    if [[ "$src_path" == "$dst_path" ]]; then
        echo "Error: Source and destination paths is the same:" \
            "$src_path; $dst_path" >&2
        exit 1
    fi

    # Exit if tar is not installed
    is_installed tar

    # Set tar_exclude
    if [ -n "$tar_exclude" ]; then
        for item in "${tar_exclude[@]}"; do
            exclude+=" --exclude $item"
        done
    else
        exclude=
    fi

    # Set options
    tar_options="${tar_options:--acf}"

    # Select filename extension by compression type.
    if [ -n "$compression" ]; then
        set_compression "$compression"
    else
        tar_ext='.tar'
    fi

    archive="${dst_path}/$(gen_backup_name "$tar_ext")"

    log "Archiving $src_path to $archive ..."

    # Run tar
    # NOTE! `exclude` and `tar_options` variables must be unquoted!
    # shellcheck disable=SC2086
    set -- tar $exclude $tar_options "$archive" "$src_path"

    log "Run command: $*"

    # shellcheck disable=SC2154
    if "$@" 2>> "$__log_file"; then
        # Append path to 'backups' array
        backups+=("$archive")
    else
        remove_if_empty "$archive"
        handle_error \
            "Error: Something went wrong when executing command:\n\t$*"
    fi
}
